/*!
 * Jodit Editor (https://xdsoft.net/jodit/)
 * License https://xdsoft.net/jodit/license.html
 * Copyright 2013-2018 Valeriy Chupurnov https://xdsoft.net
 */

/**
* Module popup edit img elements and table
* @module popup
* @params {Object} parent Jodit main object
*/
import {Jodit} from "../Jodit";
import {ToolbarCollection, ControlType} from "../modules/ToolbarCollection";
import {Config} from '../Config'
import {css, dom, offset} from "../modules/Helpers";
import {Dom} from "../modules/Dom";
import {Table} from "../modules/Table";
import {Widget} from "../modules/Widget";
import ColorPickerWidget = Widget.ColorPickerWidget;
import TabsWidget = Widget.TabsWidget;

declare module "../Config" {
    interface Config {
        popup: {[key: string]: Array<ControlType|string>}
    }
}
/**
 * @property {object} popup plugin options
 * @property {array} popup.img List of buttons toWYSIWYG the toolbar pop-up window in the image
 * @property {array} popup.table List of buttons toWYSIWYG the toolbar pop-up window at the tables
 * @example
 * ```javascript
 * new Jodit('#editor', {
 *     popup: {
 *         img: [
 *             {name: 'bin'},
 *             {
 *                  name: 'pencil',
 *                  exec: function (image) {
 *                      if (Jodit.plugins.image.open) {
 *                          Jodit.plugins.image.open.call(image);
 *                      }
 *                  }
 *              }
 *         ],
 *         table: [
 *              'splitv',
 *              'splitg', "\n",
 *              'merge', "|",
 *              {
 *                  name: 'bin',
 *                  list: {
 *                      'bin': 'Delete table',
 *                      'binrow': 'Delete row',
 *                      'bincolumn': 'Delete column',
 *                      'empty': 'Empty cell'
 *                  }
 *              }
 *         ]
 *     }
 * });
 * ```
 */
//
Config.prototype.popup = <{[key: string]: Array<ControlType|string>}>{
    a: [
        <ControlType>{
            name: 'eye',
            tooltip: 'Open link',
            exec: (editor: Jodit, current: Node) => {
                const href: string|null = (<HTMLElement>current).getAttribute('href');
                if (current && href) {
                    editor.ownerWindow.open(href);
                }
            }
        },
        {
            name: 'link',
            tooltip: 'Edit link',
            icon: 'pencil'
        },
        'unlink',
        'brush'
    ],
    img: [
        {
            name: 'bin',
            tooltip: 'Delete',
            exec: (editor: Jodit, image: Node) => {
                if (image.parentNode) {
                    image.parentNode.removeChild(image);
                }
            }
        },
        {
            name: 'pencil',
            exec: function (editor: Jodit, current: Node) {
                const tagName: string = (<HTMLElement>current).tagName.toLowerCase();
                if (tagName === 'img') {
                    editor.events.fire('openImageProperties', current);
                }
            },
            tooltip: 'Edit'
        },
        {
            name: 'valign',
            list: [
                'Top',
                'Middle',
                'Bottom'
            ],
            tooltip: 'Vertical align',
            exec: (editor: Jodit, image: HTMLImageElement, control: ControlType) => {
                const tagName: string = (<HTMLElement>image).tagName.toLowerCase();
                if (tagName !== 'img') {
                    return;
                }

                const command: string = (control.args && typeof control.args[1] === 'string') ? control.args[1].toLowerCase() : '';

                css(image, 'vertical-align', command);
            }
        },
        {
            name: 'left',
            list: [
                'Left',
                'Right',
                'Center',
                'Normal'
            ],
            exec: (editor: Jodit, image: HTMLImageElement, control: ControlType) => {
                const tagName: string = (<HTMLElement>image).tagName.toLowerCase();
                if (tagName !== 'img') {
                    return;
                }

                const clearCenterAlign = () => {
                    if (css(image, 'display') === 'block') {
                        css(image, 'display', '');
                    }
                    if (image.style.marginLeft === 'auto' && image.style.marginRight === 'auto') {
                        image.style.marginLeft =  '';
                        image.style.marginRight =  '';
                    }
                };

                const command: string = (control.args && typeof control.args[1] === 'string') ? control.args[1].toLowerCase() : '';

                if (command !== 'normal') {
                    if (['right', 'left'].indexOf(command) !== -1) {
                        css(image, 'float', command);
                        clearCenterAlign();
                    } else {
                        css(image, 'float', '');
                        css(image, {
                            'display': 'block',
                            'margin-left': 'auto',
                            'margin-right': 'auto'
                        });
                    }
                } else {
                    if (css(image, 'float') && ['right', 'left'].indexOf((<string>css(image, 'float')).toLowerCase()) !== -1) {
                        css(image, 'float', '');
                    }
                    clearCenterAlign();
                }
            },
            tooltip: 'Horizontal align'
        }
    ],
    table: [
        {
            name: 'brush',
            popup: (editor: Jodit, elm: HTMLTableElement, control: ControlType, close: Function) => {
                let $bg: HTMLElement,
                    $cl: HTMLElement,
                    $br: HTMLElement,
                    $tab: HTMLElement,
                    selected: HTMLTableCellElement[] = Table.getAllSelectedCells(elm),
                    color: string,
                    br_color: string,
                    bg_color: string;

                if (!selected.length) {
                    return false;
                }

                color = <string>css(selected[0], 'color');
                bg_color = <string>css(selected[0], 'background-color');
                br_color = <string>css(selected[0], 'border-color');


                $bg = ColorPickerWidget(editor, (value: string) => {
                    selected.forEach((cell: HTMLTableCellElement) => {
                        css(cell, 'background-color', value);
                    });
                    editor.setEditorValue();
                    // close();
                }, bg_color);

                $cl = ColorPickerWidget(editor,(value: string) => {
                    selected.forEach((cell: HTMLTableCellElement) => {
                        css(cell, 'color', value);
                    });
                    editor.setEditorValue();
                    // close();
                }, color);

                $br = ColorPickerWidget(editor,(value: string) => {
                    selected.forEach((cell: HTMLTableCellElement) => {
                        css(cell, 'border-color', value);
                    });
                    editor.setEditorValue();
                    // close();
                }, br_color);

                $tab = TabsWidget(editor, {
                    Background : $bg,
                    Text : $cl,
                    Border : $br,
                });


                return $tab;
            },
            tooltip: 'Background'
        },
        {
            name: 'valign',
            list: [
                'Top',
                'Middle',
                'Bottom'
            ],
            exec: (editor: Jodit, table: HTMLTableElement, control: ControlType) => {

                const command: string = (control.args && typeof control.args[1] === 'string') ? control.args[1].toLowerCase() : '';

                Table.getAllSelectedCells(table).forEach((cell: HTMLTableCellElement) => {
                    css(cell, 'vertical-align', command);
                })
            },
            tooltip: 'Vertical align'
        },
        {
            name: 'splitv',
            list: {
                tablesplitv: 'Split vertical',
                tablesplitg: 'Split horizontal'
            },
            tooltip: 'Split'
        },
        {
            name: 'align',
            icon: 'left'
        },
        "\n",
        {
            name: 'merge',
            command: 'tablemerge',
            tooltip: 'Merge'
        },
        {
            name: 'addcolumn',
            list: {
                tableaddcolumnbefore: 'Insert column before',
                tableaddcolumnafter: 'Insert column after'
            },
            exec: (editor: Jodit, table: HTMLTableElement, control: ControlType) => {
                const command: string = (control.args && typeof control.args[0] === 'string') ? control.args[0].toLowerCase() : '';

                editor.execCommand(command, false, table);
            },
            tooltip: 'Add column'
        },
        {
            name: 'addrow',
            list: {
                tableaddrowbefore: 'Insert row above',
                tableaddrowafter: 'Insert row below'
            },
            exec: (editor: Jodit, table: HTMLTableElement, control: ControlType) => {
                const command: string = (control.args && typeof control.args[0] === 'string') ? control.args[0].toLowerCase() : '';

                editor.execCommand(command, false, table);
            },
            tooltip: 'Add row'
        },
        {
            name: 'bin',
            list: {
                tablebin: 'Delete table',
                tablebinrow: 'Delete row',
                tablebincolumn: 'Delete column',
                tableempty: 'Empty cell'
            },
            exec: (editor: Jodit, table: HTMLTableElement, control: ControlType) => {
                const command: string = (control.args && typeof control.args[0] === 'string') ? control.args[0].toLowerCase() : '';

                editor.execCommand(command, false, table);
            },
            tooltip: 'Delete'
        }
    ]
};

export function inlinePopup(editor: Jodit) {
    let timeout: number;
    const toolbar: ToolbarCollection = new ToolbarCollection(editor),
        popup: HTMLDivElement = <HTMLDivElement> dom('<div data-editor_id="' + editor.id + '" class="jodit_toolbar_popup-inline"></div>', editor.ownerDocument),

        toggleEditor = (toggle: boolean) => {
            if (editor.container && editor.container.classList) {
                editor.container.classList.toggle('jodit_popup_active', toggle);
            }
        },

        hidePopup = (current?: HTMLElement) => {
            if (!current || !(current instanceof (<any>editor.ownerWindow).Node) || !Dom.isOrContains(popup, current)) {
                if (popup.classList) {
                    popup
                        .classList.remove('active');
                }
                toggleEditor(false);
            }
        },

        showPopup = (elm: HTMLElement, x: number, y: number) => {

            const tagName: string = elm.tagName.toLowerCase();

            if (editor.options.popup[tagName] === undefined) {
                return;
            }

            popup.innerHTML = '<span class="jodit_popup_triangle"></span>';

            toolbar.build(editor.options.popup[tagName], popup, elm);

            popup.classList
                .add('active');

            css(popup, {
                left: x + 'px',
                top: y + 'px',
            });

            popup.style.marginLeft =  -Math.round(popup.offsetWidth / 2) + 'px';

            toggleEditor(true);

            editor.events.fire('afterOpenPopup', popup,  editor.container);
        },

        delayShowPopup = (elm: HTMLElement, x: number, y: number) => {
            if (editor.options.readonly) {
                return;
            }
            clearTimeout(timeout);
            if (editor.options.observer.timeout) {
                timeout = setTimeout(showPopup.bind(editor, elm, x, y), editor.options.observer.timeout);
            } else {
                showPopup(elm, x, y);
            }
        };

    /**
     * @method init
     */

    editor.events
        .on('hidePopup afterCommand keydown resize', hidePopup)
        .on('showPopap', delayShowPopup)
        .on('afterInit', () => {
            editor.ownerDocument.body
                .appendChild(popup);

            let clickOnImage: boolean = false;

            editor.events
                .on([editor.ownerWindow, editor.editor], 'scroll', hidePopup)
                .on(popup,'mousedown', (e: MouseEvent) => {
                    e.stopPropagation();
                })
                .on(editor.editor, 'mousedown', (event: MouseEvent) => {
                    if ((<HTMLImageElement>event.target).tagName === 'IMG' || Dom.closest(<Node>event.target, 'table|a', editor.editor)) {
                        const target: HTMLImageElement|HTMLTableElement = (<HTMLImageElement>event.target).tagName === 'IMG' ? <HTMLImageElement>event.target :  <HTMLTableElement>Dom.closest(<Node>event.target, 'table|a', editor.editor);
                        const pos = offset(target, editor);
                        delayShowPopup(target, Math.round(pos.left + (target.offsetWidth / 2)), Math.round(pos.top + target.offsetHeight));
                        clickOnImage = true;
                    } else {
                        clickOnImage = false;
                    }
                })
                .on(editor.ownerWindow, 'mousedown', () => {
                    if (!clickOnImage) {
                        hidePopup();
                    }
                    clickOnImage = false;
                });
        })
        .on('beforeDestruct', () => {
            if (popup.parentNode) {
                popup.parentNode.removeChild(popup);
            }
            clearTimeout(timeout);
        });
}